#---------------------------------------------------------------------------------------------#
# Title:  Simulation Infrequent
# Purpose: Prepare infrequent data
#---------------------------------------------------------------------------------------------#

## clear the global environment
rm(list=ls())

## set the working directory
path <- "E:\\Collapsing categories\\Syntax\\02empirical_infrequent"

## load library
#install.packages("MplusAutomation") #version 0.7-2
library(MplusAutomation)

## load data
data <- read.csv("C:\\Users\\lindaliaw\\Dropbox\\Collapsing categories\\Empirical Data\\BULLY_G4_IDLINK.CSV", 
                 header = F, stringsAsFactors = F, na.strings = c(99))
data[,1] <- as.integer(gsub("﻿", "", data[,1]))
#> nrow(data)
#[1] 251493

## rename variables and select use variables
names(data) <- c("IDCNTRY","IDSCHOOL","ID","IDSTUD","CSTRAT","JKREP","JKZONE",
                 "R09A","R09B","R09C","R09D","R09E","R09F",
                 "RM02A","RM02B","RM02C","RM02D","RM02E","RM03F",
                 "RM02A_M","RM02B_M","RM02C_M","RM02D_M","RM02E_M","RM03F_M",
                 "R04","R05A","R05B","R05C","R05D","R05E","R03","GIRL","R08A","R08C",
                 "CLWGT","STUWGT")
data <- data[,c("IDCNTRY", "IDSTUD", "R09A","R09B","R09C","R09D","R09E","R09F")]

## add country id
data$CNT <- dplyr::recode(data$IDCNTRY, 
                          '31'="AZE",'40'="AUT",'48'="BHR",'51'="ARM",'72'="BOT",
                          '152'="CHL",'158'="TWN",'191'="CRO",'203'="CZE",'208'="DNK",
                          '246'="FIN",'268'="GEO",'276'="DEU",'340'="HON",'344'="HKG",
                          '348'="HUN",'364'="IRN",'372'="IRL",'380'="ITA",'392'="JAP",
                          '398'="KZH",'410'="KOR",'414'="KUW",'440'="LIT",'470'="MTA",
                          '504'="MOR",'512'="OMN",'528'="NLD",'554'="NZL",'578'="NOR",
                          '616'="POL",'620'="PTG",'634'="QTR",'642'="ROM",'643'="RUS",
                          '682'="SAU",'688'="SER",'702'="SGP",'703'="SVK",'705'="SVN",
                          '724'="ESP",'752'="SWE",'764'="THA",'784'="UAE",'788'="TUN",
                          '792'="TUR",'840'="USA",'887'="YEM")

## sample 10% of participants in each country
cnt_list <- c(unique(data$CNT))
small_data <- NULL

set.seed(43030)
for (CC in 1:length(cnt_list)){
  subdata <- data[data$CNT==cnt_list[CC],]
  small_subdata <- subdata[sample(nrow(subdata), 
                                  round(nrow(subdata)*.1),
                                  replace = TRUE), ]
  small_data <- rbind(small_data, small_subdata)
}
nrow(small_data)

## delete records with all missing responses
#empirical.na <- rowMeans( is.na(data[,paste0("R09", LETTERS[1:6])]) ) == 1  #Find records where all responses are missing
#data1 <- data[!empirical.na,] #Delete records with all missing responses
#nrow(data1)
#[1] 247338

empirical.na <- rowMeans( is.na(small_data[,paste0("R09", LETTERS[1:6])]) ) == 1  #Find records where all responses are missing
small_data <- small_data[!empirical.na,] #Delete records with all missing responses
table(empirical.na)
#empirical.na
#FALSE  TRUE 
#24783   367 

nrow(small_data)
#[1] 24783

## REMOVE COUNTRY BOT !!! 
small_data <- small_data[small_data$IDCNTRY!=72, ]
#> nrow(small_data)
#[1] 24367

prepareMplusData(small_data[,c("IDCNTRY", "IDSTUD", paste0("R09", LETTERS[1:6]))], 
                 paste0(path, "\\BULLY_G4_SMALL.dat"))