#---------------------------------------------------------------------------------------------#
# Title:  Simulation Substantive
# Purpose: Scale information
#---------------------------------------------------------------------------------------------#

## clear the global environment
rm(list=ls())

## set the working directory
path <- "E:\\Collapsing categories\\Syntax\\03simulation_substantive"

## load library
#install.packages("mirt")
#install.packages("nlme")
#install.packages("multcomp")
library(mirt) #Version 1.26.3
library("nlme")
library("multcomp")

#---------------------------------------------------------------------------------------------#
# Fitting graded response IRT model
#---------------------------------------------------------------------------------------------#

##=======================================================================##
## Baseline model: configural model
##=======================================================================##

GEN_list <- c("ci", "mi", "si")

for ( GEN_MOD  in  GEN_list  ){
  
  setwd(paste0(path, "\\03Gen_Data"))
  temp = list.files(pattern= paste0( toupper(GEN_MOD), "_dat"))
  data500 = lapply(temp, read.table)
  
  ## create a space for saving TIF
  Theta <- matrix(seq(-4,4,.001))
  
  hsd_aovtifarea_rep <- NULL
  hsd_aovtifmax_rep <- NULL
  hsd_aovtiftheta_rep <- NULL
  
  ## specify a single dataset (from n=1 till 500) 
  for (ss in 1:length(data500)){
    
    dat_4 <- data500[[ss]] #select dataset for the specific replication
    #apply(dat_4[,paste0("V", 1:6)], 2, table)
    
    dat_3a <- dat_3b <- dat_3c <- dat_2a <- dat_2b <- dat_2c <- dat_4
    
    ##============================
    ## Cat4
    ##============================
    #apply(dat_4[,paste0("V", 1:6)], 2, table) 
    
    ##============================
    ## Cat3a - 3is2
    ##============================
    dat_3a$V1[dat_3a$V1==3] <- 2
    dat_3a$V2[dat_3a$V2==3] <- 2
    dat_3a$V3[dat_3a$V3==3] <- 2
    dat_3a$V4[dat_3a$V4==3] <- 2
    dat_3a$V5[dat_3a$V5==3] <- 2
    dat_3a$V6[dat_3a$V6==3] <- 2
    #apply(dat_3a[,paste0("V", 1:6)], 2, table) 
    
    ##============================
    # Cat3b - 2is1 and 3is2
    ##============================
    dat_3b$V1[dat_3b$V1==2] <- 1
    dat_3b$V2[dat_3b$V2==2] <- 1
    dat_3b$V3[dat_3b$V3==2] <- 1
    dat_3b$V4[dat_3b$V4==2] <- 1
    dat_3b$V5[dat_3b$V5==2] <- 1
    dat_3b$V6[dat_3b$V6==2] <- 1
    
    dat_3b$V1[dat_3b$V1==3] <- 2
    dat_3b$V2[dat_3b$V2==3] <- 2
    dat_3b$V3[dat_3b$V3==3] <- 2
    dat_3b$V4[dat_3b$V4==3] <- 2
    dat_3b$V5[dat_3b$V5==3] <- 2
    dat_3b$V6[dat_3b$V6==3] <- 2
    #apply(dat_3b[,paste0("V", 1:6)], 2, table) 
    
    ##============================
    # Cat3c - 1is0, 2is1 and 3is2
    ##============================
    dat_3c$V1[dat_3c$V1==1] <- 0
    dat_3c$V2[dat_3c$V2==1] <- 0
    dat_3c$V3[dat_3c$V3==1] <- 0
    dat_3c$V4[dat_3c$V4==1] <- 0
    dat_3c$V5[dat_3c$V5==1] <- 0
    dat_3c$V6[dat_3c$V6==1] <- 0
    
    dat_3c$V1[dat_3c$V1==2] <- 1
    dat_3c$V2[dat_3c$V2==2] <- 1
    dat_3c$V3[dat_3c$V3==2] <- 1
    dat_3c$V4[dat_3c$V4==2] <- 1
    dat_3c$V5[dat_3c$V5==2] <- 1
    dat_3c$V6[dat_3c$V6==2] <- 1
    
    dat_3c$V1[dat_3c$V1==3] <- 2
    dat_3c$V2[dat_3c$V2==3] <- 2
    dat_3c$V3[dat_3c$V3==3] <- 2
    dat_3c$V4[dat_3c$V4==3] <- 2
    dat_3c$V5[dat_3c$V5==3] <- 2
    dat_3c$V6[dat_3c$V6==3] <- 2
    #apply(dat_3c[,paste0("V", 1:6)], 2, table) 
    
    ##============================
    # Cat2a - 2is1 and 3is1
    ##============================
    dat_2a$V1[dat_2a$V1==2] <- 1
    dat_2a$V2[dat_2a$V2==2] <- 1
    dat_2a$V3[dat_2a$V3==2] <- 1
    dat_2a$V4[dat_2a$V4==2] <- 1
    dat_2a$V5[dat_2a$V5==2] <- 1
    dat_2a$V6[dat_2a$V6==2] <- 1
    
    dat_2a$V1[dat_2a$V1==3] <- 1
    dat_2a$V2[dat_2a$V2==3] <- 1
    dat_2a$V3[dat_2a$V3==3] <- 1
    dat_2a$V4[dat_2a$V4==3] <- 1
    dat_2a$V5[dat_2a$V5==3] <- 1
    dat_2a$V6[dat_2a$V6==3] <- 1
    #apply(dat_2a[,paste0("V", 1:6)], 2, table) 
    
    ##============================
    # Cat2b - 1is0, 2is1 and 3is1
    ##============================
    dat_2b$V1[dat_2b$V1==1] <- 0
    dat_2b$V2[dat_2b$V2==1] <- 0
    dat_2b$V3[dat_2b$V3==1] <- 0
    dat_2b$V4[dat_2b$V4==1] <- 0
    dat_2b$V5[dat_2b$V5==1] <- 0
    dat_2b$V6[dat_2b$V6==1] <- 0
    
    dat_2b$V1[dat_2b$V1==2] <- 1
    dat_2b$V2[dat_2b$V2==2] <- 1
    dat_2b$V3[dat_2b$V3==2] <- 1
    dat_2b$V4[dat_2b$V4==2] <- 1
    dat_2b$V5[dat_2b$V5==2] <- 1
    dat_2b$V6[dat_2b$V6==2] <- 1
    
    dat_2b$V1[dat_2b$V1==3] <- 1
    dat_2b$V2[dat_2b$V2==3] <- 1
    dat_2b$V3[dat_2b$V3==3] <- 1
    dat_2b$V4[dat_2b$V4==3] <- 1
    dat_2b$V5[dat_2b$V5==3] <- 1
    dat_2b$V6[dat_2b$V6==3] <- 1
    #apply(dat_2b[,paste0("V", 1:6)], 2, table) 
    
    ##============================
    # Cat2c - 1is0, 2is0 and 3is1
    ##============================
    dat_2c$V1[dat_2c$V1==1] <- 0
    dat_2c$V2[dat_2c$V2==1] <- 0
    dat_2c$V3[dat_2c$V3==1] <- 0
    dat_2c$V4[dat_2c$V4==1] <- 0
    dat_2c$V5[dat_2c$V5==1] <- 0
    dat_2c$V6[dat_2c$V6==1] <- 0
    
    dat_2c$V1[dat_2c$V1==2] <- 0
    dat_2c$V2[dat_2c$V2==2] <- 0
    dat_2c$V3[dat_2c$V3==2] <- 0
    dat_2c$V4[dat_2c$V4==2] <- 0
    dat_2c$V5[dat_2c$V5==2] <- 0
    dat_2c$V6[dat_2c$V6==2] <- 0
    
    dat_2c$V1[dat_2c$V1==3] <- 1
    dat_2c$V2[dat_2c$V2==3] <- 1
    dat_2c$V3[dat_2c$V3==3] <- 1
    dat_2c$V4[dat_2c$V4==3] <- 1
    dat_2c$V5[dat_2c$V5==3] <- 1
    dat_2c$V6[dat_2c$V6==3] <- 1
    
    apply(dat_2c[,paste0("V", 1:6)], 2, table) 
    
    ## combine seven datasets 
    datasets <- list(dat_4, dat_3a, dat_3b, dat_3c, dat_2a, dat_2b, dat_2c)
    cond_name <- c("dat_4", "dat_3a", "dat_3b", "dat_3c", "dat_2a", "dat_2b", "dat_2c")
    
    CNTRY_TIF <- NULL #create a space for saving TIF
    
    ## fit graded IRT 
    ## areainfo: Compute the area within test or item information over a definite integral range.
    ## testinfo: Given an estimated model compute the test information.
    
    for (dd in 1:length(datasets)){
      
      subdata <- datasets[[dd]] #select dataset for the specific condition
      IDCNTRY_list <- unique(subdata[,"V7"]) # 47 countries
      
      for (i in 1:length(IDCNTRY_list)){ #calculate TIF for each country
        
        CNTRY_dataset <- subdata[subdata[,"V7"]==IDCNTRY_list[i],]  #subset data for each country
        
        # information function under mirt package
        mirt_mod <- mirt(CNTRY_dataset[,paste0("V", 1:6)], 1, itemtype="graded")
        mirt_info <- areainfo(mirt_mod, c(-3,3)) 
        
        tinfo <- as.data.frame(cbind(testinfo(mirt_mod, Theta), Theta))
        names(tinfo) <- c("testinfo","theta")
        
        mirt_max_tinfo <- tinfo[order(tinfo$testinfo, decreasing = T),][1,1]
        mirt_max_theta <- tinfo[order(tinfo$testinfo, decreasing = T),][1,2]
        
        tif <- data.frame(GEN_MOD, 
                          ss,
                          IDCNTRY_list[i], 
                          nrow(CNTRY_dataset),  
                          mirt_info$TotalInfo, 
                          mirt_max_tinfo, 
                          mirt_max_theta,
                          dd,
                          cond_name[dd])
        
        names(tif) <- c("gen_mod",
                        "reps",
                        "IDCNTRY", 
                        "COUNT", 
                        "mirt_infoarea", 
                        "mirt_max_tinfo", 
                        "mirt_max_theta",
                        "cond",
                        "cond_name")
        
        CNTRY_TIF <- rbind(CNTRY_TIF, tif)
        
      }
    }
    
    dir.create(paste0(path, "\\06Test_info", "\\CNTRY_TIF"))
    write.csv(CNTRY_TIF, paste0(path, "\\06Test_info", "\\CNTRY_TIF", 
                                "\\CNTRY_TIF_", GEN_MOD, "_", ss, ".csv"), row.names = F) 
    
    ##########################
    ##   ANOVA Test         ##
    ##########################
    # 1. Area under the curve
    tifarea <- log(CNTRY_TIF$mirt_infoarea)
    #tifarea <- (CNTRY_TIF$mirt_infoarea)
    # Add a Normal Curve 
    #h<-hist(tifarea, breaks=20, col="red", xlab="tifarea", 
    #        main="Histogram with Normal Curve",
    #        xlim=c(0,30))
    #xfit<-seq(min(tifarea),max(tifarea),length=40) 
    #yfit<-dnorm(xfit,mean=mean(tifarea),sd=sd(tifarea))
    #yfit <- yfit*diff(h$mids[1:2])*length(tifarea) 
    #lines(xfit, yfit, col="blue", lwd=2) 
    
    # Using the "log" tifarea - they are a *little* bit closer to normal. 
    # Empirical data: Using the raw tifarea - they are a *little* bit closer to normal. 
    y <- log(CNTRY_TIF$mirt_infoarea)
    x <- factor(CNTRY_TIF$cond)
    u <- factor(CNTRY_TIF$IDCNTRY)
    newdat <- data.frame(y, x, u)
    
    aovtifarea <- lme(y ~ x, random = ~1|u/x, data = newdat)
    summary(aovtifarea)
    
    # Tukey's Honest Significant Differences
    hsd_aovtifarea <- summary(glht(aovtifarea, linfct=mcp(x="Tukey")))
    hsd_aovtifarea <- as.data.frame(summary(hsd_aovtifarea)$test[ c('coefficients', 'sigma', 'tstat', 'pvalues')] )
    hsd_aovtifarea$gen_mod <- GEN_MOD
    hsd_aovtifarea$reps <- ss
    hsd_aovtifarea$pair1 <- as.character(data.frame(t(sapply(strsplit(row.names(hsd_aovtifarea), c(" - ")) , "[")))[,"X1"])
    hsd_aovtifarea$pair2 <- as.character(data.frame(t(sapply(strsplit(row.names(hsd_aovtifarea), c(" - ")) , "[")))[,"X2"])

    # 2. Max info
    tifmax <- log(CNTRY_TIF$mirt_max_tinfo)
    #tifmax <- (CNTRY_TIF$mirt_max_tinfo)
    # Add a Normal Curve 
    #h<-hist(tifmax, breaks=10, col="red", xlab="tifmax", 
    #        main="Histogram with Normal Curve") 
    #xfit<-seq(min(tifmax),max(tifmax),length=40) 
    #yfit<-dnorm(xfit,mean=mean(tifmax),sd=sd(tifmax))
    #yfit <- yfit*diff(h$mids[1:2])*length(tifmax) 
    #lines(xfit, yfit, col="blue", lwd=2) 
    
    # Using the "log" tifmax - they are a *little* bit closer to normal. 
    # Empirical data: Using the raw tifmax - they are a *little* bit closer to normal. 
    y <- log(CNTRY_TIF$mirt_max_tinfo)
    x <- factor(CNTRY_TIF$cond)
    u <- factor(CNTRY_TIF$IDCNTRY)
    newdat <- data.frame(y, x, u)
    
    aovtifmax <- lme(y ~ x, random = ~1|u/x, data = newdat)
    summary(aovtifmax)
    
    # Tukey's Honest Significant Differences
    hsd_aovtifmax <- summary(glht(aovtifmax, linfct=mcp(x="Tukey")))
    hsd_aovtifmax <- as.data.frame(summary(hsd_aovtifmax)$test[ c('coefficients', 'sigma', 'tstat', 'pvalues')] )
    hsd_aovtifmax$gen_mod <- GEN_MOD
    hsd_aovtifmax$reps <- ss
    hsd_aovtifmax$pair1 <- as.character(data.frame(t(sapply(strsplit(row.names(hsd_aovtifmax), c(" - ")) , "[")))[,"X1"])
    hsd_aovtifmax$pair2 <- as.character(data.frame(t(sapply(strsplit(row.names(hsd_aovtifmax), c(" - ")) , "[")))[,"X2"])

    # 3. Theta at max
    #tiftheta <- log(CNTRY_TIF$mirt_max_theta - min(CNTRY_TIF$mirt_max_theta) + .0001 )
    tiftheta <- (CNTRY_TIF$mirt_max_theta)
    # Add a Normal Curve 
    #h<-hist(tiftheta, breaks=10, col="red", xlab="tiftheta", 
    #        main="Histogram with Normal Curve") 
    #xfit<-seq(min(tiftheta),max(tiftheta),length=40) 
    #yfit<-dnorm(xfit,mean=mean(tiftheta),sd=sd(tiftheta))
    #yfit <- yfit*diff(h$mids[1:2])*length(tiftheta) 
    #lines(xfit, yfit, col="blue", lwd=2) 
    
    # Using the raw tifarea - they are a *little* bit closer to normal. 
    # Also raw tifarea include negative values 
    y <- (CNTRY_TIF$mirt_max_theta)
    x <- factor(CNTRY_TIF$cond)
    u <- factor(CNTRY_TIF$IDCNTRY)
    newdat <- data.frame(y, x, u)
    
    aovtiftheta <- lme(y ~ x, random = ~1|u/x, data = newdat)
    summary(aovtiftheta)
    
    # Tukey's Honest Significant Differences
    hsd_aovtiftheta <- summary(glht(aovtiftheta, linfct=mcp(x="Tukey")))
    hsd_aovtiftheta <- as.data.frame(summary(hsd_aovtiftheta)$test[ c('coefficients', 'sigma', 'tstat', 'pvalues')] )
    hsd_aovtiftheta$gen_mod <- GEN_MOD
    hsd_aovtiftheta$reps <- ss
    hsd_aovtiftheta$pair1 <- as.character(data.frame(t(sapply(strsplit(row.names(hsd_aovtiftheta), c(" - ")) , "[")))[,"X1"])
    hsd_aovtiftheta$pair2 <- as.character(data.frame(t(sapply(strsplit(row.names(hsd_aovtiftheta), c(" - ")) , "[")))[,"X2"])

    hsd_aovtifarea_rep <- rbind(hsd_aovtifarea_rep, hsd_aovtifarea) #combine HSD across 500 replication
    hsd_aovtifmax_rep <- rbind(hsd_aovtifmax_rep, hsd_aovtifmax) #combine HSD across 500 replication
    hsd_aovtiftheta_rep <- rbind(hsd_aovtiftheta_rep, hsd_aovtiftheta) #combine HSD across 500 replication

  }
  
  ################
  ## Summarize  ##
  ################
  # 1. Area under the curve
  hsd_aovtifarea_ag <- aggregate(cbind(coefficients, sigma, tstat, pvalues) ~ gen_mod + pair1 + pair2, 
                                 data=hsd_aovtifarea_rep, 
                                 function(x) c(mean = mean(x), sd = sd(x)))
  hsd_aovtifarea_ag <- do.call("data.frame", hsd_aovtifarea_ag)
  
  # 2. Max info
  hsd_aovtifmax_ag <- aggregate(cbind(coefficients, sigma, tstat, pvalues) ~ gen_mod +  pair1 + pair2, 
                                data=hsd_aovtifmax_rep, 
                                function(x) c(mean = mean(x), sd = sd(x)))
  hsd_aovtifmax_ag <- do.call("data.frame", hsd_aovtifmax_ag)
  
  # 3. Theta at max
  hsd_aovtiftheta_ag <- aggregate(cbind(coefficients, sigma, tstat, pvalues) ~ gen_mod +  pair1 + pair2, 
                                  data=hsd_aovtiftheta_rep, 
                                  function(x) c(mean = mean(x), sd = sd(x)))
  hsd_aovtiftheta_ag <- do.call("data.frame", hsd_aovtiftheta_ag)
  
}

#---------------------------------------------------------------------------------------------#
# Summarizing scale information 
#---------------------------------------------------------------------------------------------#

hsd_aov <- rbind(hsd_aovtifarea_ag, hsd_aovtifmax_ag, hsd_aovtiftheta_ag)

hsd_aov <- hsd_aov[hsd_aov$pair2 == 1, ]
hsd_aov <- hsd_aov[, c("test", "gen_mod", "pair1", "pair2",
                       "coefficients.mean", "sigma.mean", "tstat.mean", "pvalues.mean")]
hsd_aov <- hsd_aov[order(hsd_aov$test, hsd_aov$gen_mod), ]
hsd_aov$coefficients.mean <- -(hsd_aov$coefficients.mean)
hsd_aov$tstat.mean <- -(hsd_aov$tstat.mean )

write.csv(hsd_aov , paste0(path, "\\06Test_info", "\\hsd_aov.csv"), row.names = F)
