#---------------------------------------------------------------------------------------------#
# Title:  Simulation Infrequent
# Purpose: Empirical analysis
#---------------------------------------------------------------------------------------------#

## clear the global environment
rm(list=ls())

## set the working directory
path <- "E:\\Collapsing categories\\Syntax\\04simulation_infrequent"

## load library
#install.packages("MplusAutomation") #version 0.7-2
library(MplusAutomation)

#---------------------------------------------------------------------------------------------#
# Generating codes for Mplus
#---------------------------------------------------------------------------------------------#

cnt.list <- c("AZE","AUT","BHR","ARM",#"BOT",
              "CHL","TWN","CRO","CZE","DNK",
              "FIN","GEO","DEU","HON","HKG","HUN","IRN","IRL","ITA","JAP",
              "KZH","KOR","KUW","LIT","MTA","MOR","OMN","NLD","NZL","NOR",
              "POL","PTG","QTR","ROM","RUS","SAU","SER","SGP","SVK","SVN",
              "ESP","SWE","THA","UAE","TUN","TUR","USA","YEM")

inp.data <- paste("DATA: FILE IS ", paste0("'", path, "\\00Data\\BULLY_G4_SMALL.dat;"))

inp.variable <- paste("VARIABLE:",
                      "NAMES ARE IDCNTRY IDSCHOOL ID IDSTUD CSTRAT JKREP JKZONE",
                      "          R09A R09B R09C R09D R09E R09F", 
                      "          RM02A RM02B RM02C RM02D RM02E RM03F", 
                      "          RM02A_M RM02B_M RM02C_M RM02D_M RM02E_M RM03F_M",
                      "          R04 R05A R05B R05C R05D R05E R03 GIRL R08A R08C",
                      "          CLWGT STUWGT;",
                      "USEVARIABLES ARE",
                      "          R09A R09B R09C R09D R09E R09F;", 
                      "CATEGORICAL ARE", 
                      "          R09A R09B R09C R09D R09E R09F;",       
                      "MISSING ARE ALL (99);",
                      "GROUPING IS IDCNTRY (",
                      "31=AZE  40=AUT  48=BHR  51=ARM  !72=BOT",  
                      "152=CHL 158=TWN 191=CRO 203=CZE 208=DNK",
                      "246=FIN 268=GEO 276=DEU 340=HON 344=HKG 348=HUN 364=IRN 372=IRL 380=ITA 392=JAP",
                      "398=KZH 410=KOR 414=KUW 440=LIT 470=MTA 504=MOR 512=OMN 528=NLD 554=NZL 578=NOR",
                      "616=POL 620=PTG 634=QTR 642=ROM 643=RUS 682=SAU 688=SER 702=SGP 703=SVK 705=SVN",
                      "724=ESP 752=SWE 764=THA 784=UAE 788=TUN 792=TUR 840=USA 887=YEM);",
                      sep ='\n')

inp.analysis <- paste("ANALYSIS:", 
                      "   ESTIMATOR = WLSMV;", 
                      "   PARAMETERIZATION=THETA;", 
                      "   H1ITERATIONS = 3000;",
                      sep ='\n')

inp.model <- paste(
  "MODEL:",
  "   BULLY BY R09A@1 R09B R09C R09D R09E R09F;",
  "   BULLY*;",
  "   [BULLY@0];",
  "   R09A-R09F@1;",
  sep ='\n')

inp.output <- paste("OUTPUT: SAMPSTAT TECH1 TECH4 STANDARDIZED MODINDICES(3.84);") 

## create models for three levels of measurement invariance
setwd(paste0(path, "\\01Empirical"))


##=======================================================================##
## Baseline model: configural model
##=======================================================================##

title_ci <- paste("TITLE: BULLY: empirical analysis, configural invariance, small sample size")

inp.model.ci <- 
  lapply(2:48, function(i) {
    paste(paste("MODEL", paste0(cnt.list[i], ":")),
          "BULLY BY R09A@1 R09B R09C R09D R09E R09F;", 
          "[R09A$1 R09A$2 R09A$3];",
          "[R09B$1 R09B$2 R09B$3];",
          "[R09C$1 R09C$2 R09C$3];",
          "[R09D$1 R09D$2 R09D$3];",
          "[R09E$1 R09E$2 R09E$3];",
          "[R09F$1 R09F$2 R09F$3];",
          sep ='\n')
  })

inp.savedata.ci <- paste("SAVEDATA: DIFFTEST=bullysf.dif;") 

inp_ci <- paste(title_ci,
                inp.data,
                inp.variable, 
                inp.analysis, 
                inp.model,
                paste(inp.model.ci, collapse="\n"),
                inp.output,
                inp.savedata.ci,
                sep="\n")
writeLines(inp_ci, "BULLY_CI.inp")


##=======================================================================##
## Equal slopes model: metric invariance
##=======================================================================##

title_mi <- paste("TITLE: BULLY: empirical analysis, metric invariance, small sample size")

inp.model.mi <- 
  lapply(2:48, function(i) {
    paste(paste("MODEL", paste0(cnt.list[i], ":")),
          "[R09A$3];",
          "[R09B$2 R09B$3];",
          "[R09C$2 R09C$3];",
          "[R09D$2 R09D$3];",
          "[R09E$2 R09E$3];",
          "[R09F$2 R09F$3];",
          "[BULLY*];",
          "R09A-R09F*;",
          sep ='\n')
  })

inp.savedata.mi <- paste("SAVEDATA: DIFFTEST=bullyes.dif;") 

inp_mi <- paste(title_mi,
                inp.data,
                inp.variable, 
                inp.analysis, paste("DIFFTEST=bullysf.dif;"),
                inp.model,
                paste(inp.model.mi, collapse="\n"),
                inp.output,
                inp.savedata.mi,
                sep="\n")
writeLines(inp_mi, "BULLY_MI.inp")


##=======================================================================##
## Equal slopes and thresholds model: scalar invariance
##=======================================================================##

title_si <- paste("TITLE: BULLY: empirical analysis, scalar invariance, small sample size")

inp.model.si <- 
  lapply(2:48, function(i) {
    paste(paste("MODEL", paste0(cnt.list[i], ":")),
          "[BULLY*];",
          "R09A-R09F*;",
          sep ='\n')
  })

inp.savedata.si <- paste("SAVEDATA:",
                         "FILE IS bullyscalar.dat;",
                         "SAVE = FSCORES;", 
                         sep ='\n') 

inp_si <- paste(title_si,
                inp.data,
                inp.variable, paste("IDVARIABLE = IDSTUD;"), 
                inp.analysis, paste("DIFFTEST=bullyes.dif;"),
                inp.model,
                paste(inp.model.si, collapse="\n"),
                inp.output,
                inp.savedata.si,
                sep="\n")
writeLines(inp_si, "BULLY_SI.inp")


#---------------------------------------------------------------------------------------------#
# Running batches of Mplus models 
#---------------------------------------------------------------------------------------------#

runModels(target = getwd())

#---------------------------------------------------------------------------------------------#
# Extracting data from Mplus outputs
#---------------------------------------------------------------------------------------------#

bully_ci <- readModels(paste0(path,"\\01Empirical","\\bully_ci.out"))
bully_mi <- readModels(paste0(path,"\\01Empirical","\\bully_mi.out"))
bully_si <- readModels(paste0(path,"\\01Empirical","\\bully_si.out"))

#---------------------------------------------------------------------------------------------#
# Extracting model parameters
#---------------------------------------------------------------------------------------------#

## extract country names
cnt.list <- nobs[, c("CNTID", "CNT")]

## extract parameters 
ci.parameters <- bully_ci$parameters
mi.parameters <- bully_mi$parameters
si.parameters <- bully_si$parameters

##=======================================================================##
## Baseline model: configural model
##=======================================================================##

## extract "unstandardized" parameters from configural invariance 
empirical.ci.parameters <- NULL

parameters <- NULL
parameters <- ci.parameters

for ( c in 1: nrow(cnt.list)){
  parms <-  parameters$unstandardized[which(parameters$unstandardized$Group==cnt.list[c,2]),]  
  slopes <- parms[which(parms$paramHeader=="BULLY.BY"),]   
  thresholds <- parms[which(parms$paramHeader=="Thresholds"),] 
  means <- parms[which(parms$paramHeader=="Means"),] 
  variances <- parms[which(parms$paramHeader=="Variances"),] 
  res.variances <- parms[which(parms$paramHeader=="Residual.Variances"),] 
  
  empirical.ci.parms <- data.frame(matrix(c(slopes$est, 
                                            thresholds$est, 
                                            means$est, 
                                            variances$est, 
                                            res.variances$est), 
                                          nrow=1))
  colnames(empirical.ci.parms) <- c(paste0("s_",c("A","B","C","D","E","F")),
                                    paste0("t_", rep(c("A","B","C","D","E","F"), each=3), 1:3),
                                    "LV_mean",
                                    "LV_var",
                                    paste0("rv_", c("A","B","C","D","E","F")))
  
  empirical.ci.parms$CNT <- cnt.list[c,2]
  empirical.ci.parms$MI <- "configural"
  empirical.ci.parameters <- rbind(empirical.ci.parameters, empirical.ci.parms)
}

empirical.ci.parameters <- merge(nobs, empirical.ci.parameters, by="CNT")


##=======================================================================##
## Equal slopes model: metric invariance
##=======================================================================##

## extract "unstandardized" parameters from metric invariance 
empirical.mi.parameters <- NULL
parameters <- NULL
parameters <- mi.parameters

for ( c in 1: nrow(cnt.list)){
  parms <-  parameters$unstandardized[which(parameters$unstandardized$Group==cnt.list[c,2]),]  
  slopes <- parms[which(parms$paramHeader=="BULLY.BY"),]   
  thresholds <- parms[which(parms$paramHeader=="Thresholds"),] 
  means <- parms[which(parms$paramHeader=="Means"),] 
  variances <- parms[which(parms$paramHeader=="Variances"),] 
  res.variances <- parms[which(parms$paramHeader=="Residual.Variances"),] 
  
  empirical.mi.parms <- data.frame(matrix(c(slopes$est, thresholds$est, means$est, variances$est, res.variances$est), 
                                          nrow=1))
  colnames(empirical.mi.parms) <- c(paste0("s_",c("A","B","C","D","E","F")),
                                    paste0("t_", rep(c("A","B","C","D","E","F"), each=3), 1:3),
                                    "LV_mean",
                                    "LV_var",
                                    paste0("rv_", c("A","B","C","D","E","F")))
  
  empirical.mi.parms$CNT <- cnt.list[c,2]
  empirical.mi.parms$MI <- "metric"
  empirical.mi.parameters <- rbind(empirical.mi.parameters, empirical.mi.parms)
}

empirical.mi.parameters <- merge(nobs, empirical.mi.parameters, by="CNT")


##=======================================================================##
## Equal slopes and thresholds model: scalar invariance
##=======================================================================##

## extract "unstandardized" parameters from scalar invariance 
empirical.si.parameters <- NULL
parameters <- NULL
parameters <- si.parameters

for ( c in 1: nrow(cnt.list)){
  parms <-  parameters$unstandardized[which(parameters$unstandardized$Group==cnt.list[c,2]),]  
  slopes <- parms[which(parms$paramHeader=="BULLY.BY"),]   
  thresholds <- parms[which(parms$paramHeader=="Thresholds"),] 
  means <- parms[which(parms$paramHeader=="Means"),] 
  variances <- parms[which(parms$paramHeader=="Variances"),] 
  res.variances <- parms[which(parms$paramHeader=="Residual.Variances"),] 
  
  empirical.si.parms <- data.frame(matrix(c(slopes$est, thresholds$est, means$est, variances$est, res.variances$est), 
                                          nrow=1))
  colnames(empirical.si.parms) <- c(paste0("s_",c("A","B","C","D","E","F")),
                                    paste0("t_", rep(c("A","B","C","D","E","F"), each=3), 1:3),
                                    "LV_mean",
                                    "LV_var",
                                    paste0("rv_", c("A","B","C","D","E","F")))
  
  empirical.si.parms$CNT <- cnt.list[c,2]
  empirical.si.parms$MI <- "scalar"
  empirical.si.parameters <- rbind(empirical.si.parameters, empirical.si.parms)
}

empirical.si.parameters <- merge(nobs, empirical.si.parameters, by="CNT")

save(empirical.ci.parameters, 
     file = paste0(path,"\\01Empirical", "\\empirical.ci.parameters", ".RData"))
save(empirical.mi.parameters, 
     file = paste0(path,"\\01Empirical", "\\empirical.mi.parameters", ".RData"))
save(empirical.si.parameters, 
     file = paste0(path,"\\01Empirical", "\\empirical.si.parameters", ".RData"))

gen_est <- cbind(apply(empirical.ci.parameters[, c(4:35)], 2, mean), 
                 apply(empirical.ci.parameters[, c(4:35)], 2, sd),
                 apply(empirical.mi.parameters[, c(4:35)], 2, mean), 
                 apply(empirical.mi.parameters[, c(4:35)], 2, sd),
                 apply(empirical.si.parameters[, c(4:35)], 2, mean), 
                 apply(empirical.si.parameters[, c(4:35)], 2, sd))
write.csv(gen_est, paste0(path, "\\01Empirical", "\\gen_est.csv"))