#---------------------------------------------------------------------------------------------#
# Title:  Simulation Infrequent
# Purpose: Data generation
#---------------------------------------------------------------------------------------------#

## clear the global environment
rm(list=ls())

## set the working directory
path <- "E:\\Collapsing categories\\Syntax\\04simulation_infrequent"

## load library
#install.packages("MplusAutomation") #version 0.7-2
library(MplusAutomation)

#---------------------------------------------------------------------------------------------#
# Generating codes for data generation
#---------------------------------------------------------------------------------------------#

## country names 
cnt.list <- nobs[,c("CNTID", "CNT")]

## load empirical parameters 
load(file = paste0(path,"\\01Empirical", "\\empirical.ci.parameters", ".RData"))
load(file = paste0(path,"\\01Empirical", "\\empirical.mi.parameters", ".RData"))
load(file = paste0(path,"\\01Empirical", "\\empirical.si.parameters", ".RData"))


##=======================================================================##
## Baseline model: configural model
##=======================================================================##

p <- parms <- empirical.ci.parameters
p <- parms <- parms[order(parms$CNTID),]

inp <- NULL
inp <- paste(
  "TITLE: Testing: data generation, configural invariance, small sample size",
  "MONTECARLO:",
  "   NAMES = V1-V6;",
  "   GENERATE = V1-V6(3);",
  "   CATEGORICAL = V1-V6;",
  paste0("   NGROUPS = ", length(parms$Freq), ";"),
  "   NOBS = ", paste(strwrap(x = paste(parms$Freq, collapse=" "), width = 40, simplify = T), collapse = "\n"), ";",
  "   SEED = 91716;",
  "   NREPS = 5;", 
  "   REPSAVE = ALL;",
  "   SAVE = CI_dat*.dat;", 
  "ANALYSIS:", 
  "   ESTIMATOR = WLSMV;", 
  "   PARAMETERIZATION=THETA;", 
  "   H1ITERATIONS = 3000;",
  sep ='\n')

inp2 <- 
  lapply(1:nrow(p), function(i) {
    paste(paste(paste0("MODEL POPULATION-g", i, ":"), paste0("!", parms$CNT[i])),
          "! Loadings",
          paste("   ", "BULLY BY", paste0("V",1:6, "*", p[i, paste0("s_", LETTERS[1:6])], collapse = " "), ";"), 
          "! Thresholds",
          paste0("   ", "[", paste0("V1$",1:3, "*", p[i, paste0("t_A", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V2$",1:3, "*", p[i, paste0("t_B", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V3$",1:3, "*", p[i, paste0("t_C", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V4$",1:3, "*", p[i, paste0("t_D", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V5$",1:3, "*", p[i, paste0("t_E", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V6$",1:3, "*", p[i, paste0("t_F", 1:3)], collapse = " "), "]", ";"), 
          "! Latent Mean",
          paste0("   ", "[BULLY@", p[i,"LV_mean"], "]", ";"),
          "! Latent Variance",  
          paste0("   ", "BULLY@", p[i,"LV_var"], ";"),
          "!Residual Variance",
          paste0("   ", paste0("V",1:6, "*", p[i, paste0("rv_", LETTERS[1:6])], collapse = " "), ";"), 
          sep ='\n')
  })

inp3 <- 
  lapply(1:nrow(p), function(i) {
    paste(paste(paste0("MODEL g", i, ":"), paste0("!", parms$CNT[i])),
          "! Loadings",
          paste("   ", "BULLY BY", paste0("V",1:6, "*", p[i, paste0("s_", LETTERS[1:6])], collapse = " "), ";"), 
          "! Thresholds",
          paste0("   ", "[", paste0("V1$",1:3, "*", p[i, paste0("t_A", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V2$",1:3, "*", p[i, paste0("t_B", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V3$",1:3, "*", p[i, paste0("t_C", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V4$",1:3, "*", p[i, paste0("t_D", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V5$",1:3, "*", p[i, paste0("t_E", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V6$",1:3, "*", p[i, paste0("t_F", 1:3)], collapse = " "), "]", ";"), 
          "! Latent Mean",
          paste0("   ", "[BULLY@", p[i,"LV_mean"], "]", ";"),
          "! Latent Variance",  
          paste0("   ", "BULLY@", p[i,"LV_var"], ";"),
          "!Residual Variance",
          paste0("   ", paste0("V",1:6, "*", p[i, paste0("rv_", LETTERS[1:6])], collapse = " "), ";"), 
          sep ='\n')
  })

inp <- paste(inp, gsub("-g1:", ":", paste(inp2, collapse="\n")), gsub("MODEL g1:", "MODEL:", paste(inp3, collapse="\n")), sep="\n")
writeLines(inp, paste0(path, "\\02Generation", "\\gen_CI.inp"))


##=======================================================================##
## Equal slopes model: metric invariance
##=======================================================================##

p <- parms <- empirical.mi.parameters #update to metric invariance
p <- parms <- parms[order(parms$CNTID),]

inp <- NULL
inp <- paste(
  "TITLE: Testing: data generation, metric invariance, small sample size",
  "MONTECARLO:",
  "   NAMES = V1-V6;",
  "   GENERATE = V1-V6(3);",
  "   CATEGORICAL = V1-V6;",
  paste0("   NGROUPS = ", length(parms$Freq), ";"),
  "   NOBS = ", paste(strwrap(x = paste(parms$Freq, collapse=" "), width = 40, simplify = T), collapse = "\n"), ";",
  "   SEED = 91716;",
  "   NREPS = 5;", 
  "   REPSAVE = ALL;",
  "   SAVE = MI_dat*.dat;", 
  "ANALYSIS:", 
  "   ESTIMATOR = WLSMV;", 
  "   PARAMETERIZATION=THETA;", 
  "   H1ITERATIONS = 3000;",
  sep ='\n')

inp2 <- 
  lapply(1:nrow(p), function(i) {
    paste(paste(paste0("MODEL POPULATION-g", i, ":"), paste0("!", parms$CNT[i])),
          "! Loadings",
          paste("   ", "BULLY BY", paste0("V",1:6, "*", p[i, paste0("s_", LETTERS[1:6])], collapse = " "), ";"), 
          "! Thresholds",
          paste0("   ", "[", paste0("V1$",1:3, "*", p[i, paste0("t_A", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V2$",1:3, "*", p[i, paste0("t_B", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V3$",1:3, "*", p[i, paste0("t_C", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V4$",1:3, "*", p[i, paste0("t_D", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V5$",1:3, "*", p[i, paste0("t_E", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V6$",1:3, "*", p[i, paste0("t_F", 1:3)], collapse = " "), "]", ";"), 
          "! Latent Mean",
          paste0("   ", "[BULLY@", p[i,"LV_mean"], "]", ";"),
          "! Latent Variance",  
          paste0("   ", "BULLY@", p[i,"LV_var"], ";"),
          "!Residual Variance",
          paste0("   ", paste0("V",1:6, "*", p[i, paste0("rv_", LETTERS[1:6])], collapse = " "), ";"), 
          sep ='\n')
  })

inp3 <- 
  lapply(1:nrow(p), function(i) {
    paste(paste(paste0("MODEL g", i, ":"), paste0("!", parms$CNT[i])),
          "! Loadings",
          paste("   ", "BULLY BY", paste0("V",1:6, "*", p[i, paste0("s_", LETTERS[1:6])], collapse = " "), ";"), 
          "! Thresholds",
          paste0("   ", "[", paste0("V1$",1:3, "*", p[i, paste0("t_A", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V2$",1:3, "*", p[i, paste0("t_B", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V3$",1:3, "*", p[i, paste0("t_C", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V4$",1:3, "*", p[i, paste0("t_D", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V5$",1:3, "*", p[i, paste0("t_E", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V6$",1:3, "*", p[i, paste0("t_F", 1:3)], collapse = " "), "]", ";"), 
          "! Latent Mean",
          paste0("   ", "[BULLY@", p[i,"LV_mean"], "]", ";"),
          "! Latent Variance",  
          paste0("   ", "BULLY@", p[i,"LV_var"], ";"),
          "!Residual Variance",
          paste0("   ", paste0("V",1:6, "*", p[i, paste0("rv_", LETTERS[1:6])], collapse = " "), ";"), 
          sep ='\n')
  })

inp <- paste(inp, gsub("-g1:", ":", paste(inp2, collapse="\n")), gsub("MODEL g1:", "MODEL:", paste(inp3, collapse="\n")), sep="\n")
writeLines(inp, paste0(path, "\\02Generation", "\\gen_MI.inp"))


##=======================================================================##
## Equal slopes and thresholds model: scalar invariance
##=======================================================================##

p <- parms <- empirical.si.parameters #update to scalar invariance
p <- parms <- parms[order(parms$CNTID),]

inp <- NULL
inp <- paste(
  "TITLE: Testing: data generation, scalar invariance, small sample size",
  "MONTECARLO:",
  "   NAMES = V1-V6;",
  "   GENERATE = V1-V6(3);",
  "   CATEGORICAL = V1-V6;",
  paste0("   NGROUPS = ", length(parms$Freq), ";"),
  "   NOBS = ", paste(strwrap(x = paste(parms$Freq, collapse=" "), width = 40, simplify = T), collapse = "\n"), ";",
  "   SEED = 91716;",
  "   NREPS = 5;", 
  "   REPSAVE = ALL;",
  #paste0("   SAVE =", path,"\\03Gen_Data", "\\", "SI_dat*.dat;"), 
  "   SAVE = SI_dat*.dat;", 
  "ANALYSIS:", 
  "   ESTIMATOR = WLSMV;", 
  "   PARAMETERIZATION=THETA;", 
  "   H1ITERATIONS = 3000;",
  sep ='\n')

inp2 <- 
  lapply(1:nrow(p), function(i) {
    paste(paste(paste0("MODEL POPULATION-g", i, ":"), paste0("!", parms$CNT[i])),
          "! Loadings",
          paste("   ", "BULLY BY", paste0("V",1:6, "*", p[i, paste0("s_", LETTERS[1:6])], collapse = " "), ";"), 
          "! Thresholds",
          paste0("   ", "[", paste0("V1$",1:3, "*", p[i, paste0("t_A", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V2$",1:3, "*", p[i, paste0("t_B", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V3$",1:3, "*", p[i, paste0("t_C", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V4$",1:3, "*", p[i, paste0("t_D", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V5$",1:3, "*", p[i, paste0("t_E", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V6$",1:3, "*", p[i, paste0("t_F", 1:3)], collapse = " "), "]", ";"), 
          "! Latent Mean",
          paste0("   ", "[BULLY@", p[i,"LV_mean"], "]", ";"),
          "! Latent Variance",  
          paste0("   ", "BULLY@", p[i,"LV_var"], ";"),
          "!Residual Variance",
          paste0("   ", paste0("V",1:6, "*", p[i, paste0("rv_", LETTERS[1:6])], collapse = " "), ";"), 
          sep ='\n')
  })

inp3 <- 
  lapply(1:nrow(p), function(i) {
    paste(paste(paste0("MODEL g", i, ":"), paste0("!", parms$CNT[i])),
          "! Loadings",
          paste("   ", "BULLY BY", paste0("V",1:6, "*", p[i, paste0("s_", LETTERS[1:6])], collapse = " "), ";"), 
          "! Thresholds",
          paste0("   ", "[", paste0("V1$",1:3, "*", p[i, paste0("t_A", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V2$",1:3, "*", p[i, paste0("t_B", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V3$",1:3, "*", p[i, paste0("t_C", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V4$",1:3, "*", p[i, paste0("t_D", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V5$",1:3, "*", p[i, paste0("t_E", 1:3)], collapse = " "), "]", ";"), 
          paste0("   ", "[", paste0("V6$",1:3, "*", p[i, paste0("t_F", 1:3)], collapse = " "), "]", ";"), 
          "! Latent Mean",
          paste0("   ", "[BULLY@", p[i,"LV_mean"], "]", ";"),
          "! Latent Variance",  
          paste0("   ", "BULLY@", p[i,"LV_var"], ";"),
          "!Residual Variance",
          paste0("   ", paste0("V",1:6, "*", p[i, paste0("rv_", LETTERS[1:6])], collapse = " "), ";"), 
          sep ='\n')
  })

inp <- paste(inp, gsub("-g1:", ":", paste(inp2, collapse="\n")), gsub("MODEL g1:", "MODEL:", paste(inp3, collapse="\n")), sep="\n")
writeLines(inp, paste0(path, "\\02Generation", "\\gen_SI.inp"))


#---------------------------------------------------------------------------------------------#
# Running batches of Mplus 
#---------------------------------------------------------------------------------------------#

runModels(target = paste0(path, "\\02Generation"))

