#---------------------------------------------------------------------------------------------#
# Title:  Simulation Infrequent
# Purpose: Measurement invariance tests
#---------------------------------------------------------------------------------------------#

## clear the global environment
rm(list=ls())

## set the working directory
path <- "E:\\Collapsing categories\\Syntax\\04simulation_infrequent"

## load library
#install.packages("MplusAutomation") #version 0.7-2
library(MplusAutomation)

cnt.list <- c("AZE","AUT","BHR","ARM",#"BOT",
              "CHL","TWN","CRO","CZE","DNK",
              "FIN","GEO","DEU","HON","HKG","HUN","IRN","IRL","ITA","JAP",
              "KZH","KOR","KUW","LIT","MTA","MOR","OMN","NLD","NZL","NOR",
              "POL","PTG","QTR","ROM","RUS","SAU","SER","SGP","SVK","SVN",
              "ESP","SWE","THA","UAE","TUN","TUR","USA","YEM")

#---------------------------------------------------------------------------------------------#
# Define categories strategy
#---------------------------------------------------------------------------------------------#

####################################################
# 0- .4  ##  Four    ###    1  | 2  | 3  | 4     ###     
####################################################
# 1- CAT3a ##  Three   ###    1  | 2  |  34     ###
# 2- CAT3b ##          ###    1  | 23 |  4      ###
# 3- CAT3c ##          ###    12 | 3  |  4      ###
####################################################  
# 4- CAT2a ##  Two     ###    1    | 234        ###
# 5- CAT2b ##          ###    12   | 34         ###
# 6- CAT2c ##          ###    123  | 4          ###
####################################################

#---------------------------------------------------------------------------------------------#
# Generating codes for Mplus
#---------------------------------------------------------------------------------------------#

Collapsing <- list(CAT4 = paste0(paste(paste0("IF R09", LETTERS[1:6], " EQ 3 THEN R09", LETTERS[1:6], "C = 3"), 
                                       paste0("IF R09", LETTERS[1:6], " EQ 2 THEN R09", LETTERS[1:6], "C = 2"),
                                       paste0("IF R09", LETTERS[1:6], " EQ 1 THEN R09", LETTERS[1:6], "C = 1"), 
                                       paste0("IF R09", LETTERS[1:6], " EQ 0 THEN R09", LETTERS[1:6], "C = 0"),
                                       sep =';\n', collapse=";\n"), ";"),
                   CAT3a = paste0(paste(paste0("IF R09", LETTERS[1:6], " EQ 3 THEN R09", LETTERS[1:6], "C = 2"), 
                                        paste0("IF R09", LETTERS[1:6], " EQ 2 THEN R09", LETTERS[1:6], "C = 2"),
                                        paste0("IF R09", LETTERS[1:6], " EQ 1 THEN R09", LETTERS[1:6], "C = 1"), 
                                        paste0("IF R09", LETTERS[1:6], " EQ 0 THEN R09", LETTERS[1:6], "C = 0"),
                                        sep =';\n', collapse=";\n"), ";"),
                   CAT3b = paste0(paste(paste0("IF R09", LETTERS[1:6], " EQ 3 THEN R09", LETTERS[1:6], "C = 2"), 
                                        paste0("IF R09", LETTERS[1:6], " EQ 2 THEN R09", LETTERS[1:6], "C = 1"),
                                        paste0("IF R09", LETTERS[1:6], " EQ 1 THEN R09", LETTERS[1:6], "C = 1"), 
                                        paste0("IF R09", LETTERS[1:6], " EQ 0 THEN R09", LETTERS[1:6], "C = 0"),
                                        sep =';\n', collapse=";\n"), ";"),
                   CAT3c = paste0(paste(paste0("IF R09", LETTERS[1:6], " EQ 3 THEN R09", LETTERS[1:6], "C = 2"), 
                                        paste0("IF R09", LETTERS[1:6], " EQ 2 THEN R09", LETTERS[1:6], "C = 1"),
                                        paste0("IF R09", LETTERS[1:6], " EQ 1 THEN R09", LETTERS[1:6], "C = 0"), 
                                        paste0("IF R09", LETTERS[1:6], " EQ 0 THEN R09", LETTERS[1:6], "C = 0"),
                                        sep =';\n', collapse=";\n"), ";"),
                   CAT2a = paste0(paste(paste0("IF R09", LETTERS[1:6], " EQ 3 THEN R09", LETTERS[1:6], "C = 1"), 
                                        paste0("IF R09", LETTERS[1:6], " EQ 2 THEN R09", LETTERS[1:6], "C = 1"),
                                        paste0("IF R09", LETTERS[1:6], " EQ 1 THEN R09", LETTERS[1:6], "C = 1"), 
                                        paste0("IF R09", LETTERS[1:6], " EQ 0 THEN R09", LETTERS[1:6], "C = 0"),
                                        sep =';\n', collapse=";\n"), ";"),
                   CAT2b = paste0(paste(paste0("IF R09", LETTERS[1:6], " EQ 3 THEN R09", LETTERS[1:6], "C = 1"), 
                                        paste0("IF R09", LETTERS[1:6], " EQ 2 THEN R09", LETTERS[1:6], "C = 1"),
                                        paste0("IF R09", LETTERS[1:6], " EQ 1 THEN R09", LETTERS[1:6], "C = 0"), 
                                        paste0("IF R09", LETTERS[1:6], " EQ 0 THEN R09", LETTERS[1:6], "C = 0"),
                                        sep =';\n', collapse=";\n"), ";"),
                   CAT2c = paste0(paste(paste0("IF R09", LETTERS[1:6], " EQ 3 THEN R09", LETTERS[1:6], "C = 1"), 
                                        paste0("IF R09", LETTERS[1:6], " EQ 2 THEN R09", LETTERS[1:6], "C = 0"),
                                        paste0("IF R09", LETTERS[1:6], " EQ 1 THEN R09", LETTERS[1:6], "C = 0"), 
                                        paste0("IF R09", LETTERS[1:6], " EQ 0 THEN R09", LETTERS[1:6], "C = 0"),
                                        sep =';\n', collapse=";\n"), ";"))

reps <- 500

for (k in 1: reps){
  
  #####################
  ##    TITLE LINE   ##
  #####################
  title <- list(ci_CAT4 = paste("TITLE: Small size: CI, 4cat", "rep", k),
                mi_CAT4 = paste("TITLE: Small size: MI, 4cat", "rep", k),
                si_CAT4 = paste("TITLE: Small size: SI, 4cat", "rep", k),
                ci_CAT3a = paste("TITLE: Small size: CI, 3cat, 3is2", "rep", k),
                mi_CAT3a = paste("TITLE: Small size: MI, 3cat, 3is2", "rep", k),
                si_CAT3a = paste("TITLE: Small size: SI, 3cat, 3is2", "rep", k),
                ci_CAT3b = paste("TITLE: Small size: CI, 3cat, 2is1 and 3is2", "rep", k),
                mi_CAT3b = paste("TITLE: Small size: MI, 3cat, 2is1 and 3is2", "rep", k),
                si_CAT3b = paste("TITLE: Small size: SI, 3cat, 2is1 and 3is2", "rep", k),
                ci_CAT3c = paste("TITLE: Small size: CI, 3cat, 1is0, 2is1 and 3is2", "rep", k),
                mi_CAT3c = paste("TITLE: Small size: MI, 3cat, 1is0, 2is1 and 3is2", "rep", k),
                si_CAT3c = paste("TITLE: Small size: SI, 3cat, 1is0, 2is1 and 3is2", "rep", k),
                ci_CAT2a = paste("TITLE: Small size: CI, 2cat, 2is1 and 3is1", "rep", k),
                mi_CAT2a = paste("TITLE: Small size: MI, 2cat, 2is1 and 3is1", "rep", k),
                si_CAT2a = paste("TITLE: Small size: SI, 2cat, 2is1 and 3is1", "rep", k),
                ci_CAT2b = paste("TITLE: Small size: CI, 2cat, 1is0, 2is1 and 3is1", "rep", k),
                mi_CAT2b = paste("TITLE: Small size: MI, 2cat, 1is0, 2is1 and 3is1", "rep", k),
                si_CAT2b = paste("TITLE: Small size: SI, 2cat, 1is0, 2is1 and 3is1", "rep", k),
                ci_CAT2c = paste("TITLE: Small size: CI, 2cat, 1is0, 2is0 and 3is1", "rep", k),
                mi_CAT2c = paste("TITLE: Small size: MI, 2cat, 1is0, 2is0 and 3is1", "rep", k),
                si_CAT2c = paste("TITLE: Small size: SI, 2cat, 1is0, 2is0 and 3is1", "rep", k))
  
  #####################
  ##    DATA LINE    ##
  #####################
  inp.data <- list(ci =  paste0("DATA: FILE IS '", paste0(path_data,"\\03Gen_Data", "\\CI_dat", k, ".dat"), "';"),
                   mi =  paste0("DATA: FILE IS '", paste0(path_data,"\\03Gen_Data", "\\MI_dat", k, ".dat"), "';"),
                   si =  paste0("DATA: FILE IS '", paste0(path_data,"\\03Gen_Data", "\\SI_dat", k, ".dat"), "';"))
  
  #####################
  ##  VARAIBLE LINE  ##
  #####################
  inp.variable <- paste("VARIABLE:",
                        "NAMES ARE R09A R09B R09C R09D R09E R09F CNT;",
                        "USEVARIABLES ARE",
                        "          R09AC R09BC R09CC R09DC R09EC R09FC;", 
                        "CATEGORICAL ARE", 
                        "          R09AC R09BC R09CC R09DC R09EC R09FC;",       
                        "MISSING ARE ALL (99);",
                        "GROUPING IS CNT (",
                        paste(strwrap(x = paste( paste0(1:47, "=", cnt.list), collapse=" "), 
                                      width = 40, simplify = T), collapse = "\n"),
                        ");",
                        sep ='\n')
  
  #####################
  ##   DEFINE LINE   ##
  #####################
  inp.define <- list(CAT4 = paste("!4 categories, no collapsing", "DEFINE:",  Collapsing[["CAT4"]], sep ='\n'),
                     CAT3a = paste("!3 categories, 3is2", "DEFINE:", Collapsing[["CAT3a"]], sep ='\n'),
                     CAT3b = paste("!3 categories, 2is1 and 3is2", "DEFINE:", Collapsing[["CAT3b"]], sep ='\n'),
                     CAT3c = paste("!3 categories, 1is0, 2is1 and 3is2", "DEFINE:", Collapsing[["CAT3c"]], sep ='\n'),
                     CAT2a = paste("!2 categories, 2is1 and 3is1", "DEFINE:", Collapsing[["CAT2a"]], sep ='\n'),
                     CAT2b = paste("!2 categories, 1is0, 2is1 and 3is1", "DEFINE:", Collapsing[["CAT2b"]], sep ='\n'),
                     CAT2c = paste("!2 categories, 1is0, 2is0 and 3is1", "DEFINE:", Collapsing[["CAT2c"]], sep ='\n'))
  
  #####################
  ##  ANALYSIS LINE  ##
  #####################
  inp.analysis3 <- paste("ANALYSIS:", 
                        "   ESTIMATOR = WLSMV;", 
                        "   PARAMETERIZATION=THETA;", 
                        "   H1ITERATIONS = 3000;",
                        "   MODEL = CONFIGURAL METRIC SCALAR;",
                        sep ='\n')

  inp.analysis2 <- paste("ANALYSIS:", 
                        "   ESTIMATOR = WLSMV;", 
                        "   PARAMETERIZATION=THETA;", 
                        "   H1ITERATIONS = 3000;",
                        "   MODEL = CONFIGURAL SCALAR;",
                        sep ='\n')
  
  
  #####################
  ##    MODEL LINE   ##
  #####################
  inp.model <- paste(
    "MODEL:",
    "   BULLY BY R09AC@1 R09BC R09CC R09DC R09EC R09FC;",
    sep ='\n')

  inp.model.group <- lapply(2:47, function(i) {
      paste(paste("MODEL", paste0(cnt.list[i], ":")),
            "R09AC* R09BC* R09CC* R09DC* R09EC* R09FC*;", 
            sep ='\n')})
    
  # create syntaxes for Mplus
  IV.list <- c("ci","mi","si")
  CAT.list <- paste0("CAT", c("4", "3a","3b","3c","2a","2b","2c"))
  
  for(i in IV.list) {
    
    for(j in CAT.list) {
      
      input <- paste(
        title[[paste0(i, "_", j)]],
        inp.data[[i]],
        inp.variable,
        inp.define[[j]],
        
        if(j %in% c("CAT2a", "CAT2b", "CAT2c")) {
          inp.analysis2} else 
          {  inp.analysis3},
        inp.model,
        paste(inp.model.group, collapse="\n"),
        sep="\n")
      
      mod_path <- paste0(path, "\\04Analysis_Convenience", "\\", j)
      dir.create(path = mod_path, showWarnings = F)
      
      writeLines(text = input, 
                 con =  paste0(mod_path, "\\", i, "_", j, "_rep_", k, ".inp"))
    
      }
  }
}

#---------------------------------------------------------------------------------------------#
# Running batches of Mplus models 
#---------------------------------------------------------------------------------------------#

runModels(target = paste0(path, "\\04Analysis_Convenience"), recursive = T) 


#---------------------------------------------------------------------------------------------#
# Extracting data from Mplus outputs
#---------------------------------------------------------------------------------------------#

SimulateOutput <- readModels(target = paste0(path, "\\04Analysis_Convenience"), recursive = T) 
save(SimulateOutput, file = paste0(path, "\\05Mplus_output", "\\SimulateOutput", ".RData"))
#load(file = paste0(path, "\\05Mplus_output", "\\SimulateOutput", ".RData"))


#---------------------------------------------------------------------------------------------#
# Extracting model summary statistics
#---------------------------------------------------------------------------------------------#

fits <- do.call("rbind", sapply(SimulateOutput , "[", "summaries"))

fits$gen <- as.character(data.frame(t(sapply(strsplit(fits$Filename, c("_")) , "[")))[,"X1"])
fits$cat <- as.character(data.frame(t(sapply(strsplit(fits$Filename, c("_")) , "[")))[,"X2"])
fits$cond <- gsub("cat", "", fits$cat)
fits$rep <- as.numeric(gsub(".out", "", as.character(data.frame(t(sapply(strsplit(fits$Filename, c("_")) , "[")))[,"X4"])))
rownames(fits) <- NULL
fits$est <- fits$Model
fits$RMSEA <- fits$RMSEA_Estimate

fits <- fits[, c("gen", "cond", "rep", "est",
                 "Parameters", 
                 "ChiSqM_Value", "ChiSqM_DF", "ChiSqM_PValue",
                 "CFI", "TLI", "RMSEA")]

# only keep the full convergence replications
GEN_MODEL <- c("ci", "mi", "si")
COND <- c("2a", "2b", "2c", "3a", "3b", "3c", "4")
fits_convg <- NULL

for (gg in GEN_MODEL){
  
  for (cc in COND) {
    
    for (kk in 1:500) {
      
      subdata <- fits[fits$gen == gg & fits$cond == cc & fits$rep == kk, ]
      
      if ( (unique(subdata$cond) %in% c("2a", "2b", "2c") & nrow(subdata) == 2) == TRUE) {
        subdata <- subdata 
        subdata[subdata$est == "CONFIGURAL MODEL", "delta_CFI"]  <- subdata[subdata$est == "CONFIGURAL MODEL", "delta_RMSEA"] <- 0
        subdata[subdata$est == "SCALAR MODEL", "delta_CFI"] <- 
          subdata[subdata$est == "SCALAR MODEL", "CFI"] - subdata[subdata$est == "CONFIGURAL MODEL", "CFI"]
        subdata[subdata$est == "SCALAR MODEL", "delta_RMSEA"] <- 
          subdata[subdata$est == "SCALAR MODEL", "RMSEA"] - subdata[subdata$est == "CONFIGURAL MODEL", "RMSEA"]
      } else if ( (unique(subdata$cond) %in% c("3a", "3b", "3c", "4") & nrow(subdata) == 3) == TRUE) {
        subdata <- subdata 
        subdata[subdata$est == "CONFIGURAL MODEL", "delta_CFI"]  <- subdata[subdata$est == "CONFIGURAL MODEL", "delta_RMSEA"] <- 0
        subdata[subdata$est == "METRIC MODEL", "delta_CFI"] <- 
          subdata[subdata$est == "METRIC MODEL", "CFI"] - subdata[subdata$est == "CONFIGURAL MODEL", "CFI"]
        subdata[subdata$est == "SCALAR MODEL", "delta_CFI"] <- 
          subdata[subdata$est == "SCALAR MODEL", "CFI"] - subdata[subdata$est == "METRIC MODEL", "CFI"]
        subdata[subdata$est == "METRIC MODEL", "delta_RMSEA"] <- 
          subdata[subdata$est == "METRIC MODEL", "RMSEA"] - subdata[subdata$est == "CONFIGURAL MODEL", "RMSEA"]
        subdata[subdata$est == "SCALAR MODEL", "delta_RMSEA"] <- 
          subdata[subdata$est == "SCALAR MODEL", "RMSEA"] - subdata[subdata$est == "METRIC MODEL", "RMSEA"]
      } else {
        subdata <- NULL 
      }
      
      fits_convg <- rbind(fits_convg, subdata)
      
    }
  }
}

all_fits <- aggregate(cbind(Parameters,
                            ChiSqM_Value, ChiSqM_DF, ChiSqM_PValue, 
                            CFI, TLI, RMSEA,
                            delta_CFI, delta_RMSEA)
                      ~ gen + cond + est ,
                      data = fits_convg, mean)

agg_n <- aggregate(cbind(Parameters)
                   ~ gen + cond + est ,
                   data = fits_convg, length)
colnames(agg_n) <- c("gen", "cond", "est", "#ofconvg")

all_fits <- merge(all_fits, agg_n, by = c("gen", "cond", "est"))
all_fits$condition <- dplyr::recode(all_fits$cond, "4" = 1, "3a" = 2, "3b" = 3, "3c" = 4,
                                    "2a" = 5, "2b" = 6, "2c" = 7)
all_fits$est <- dplyr::recode(all_fits$est, 
                              "CONFIGURAL MODEL" = "Baseline", 
                              "METRIC MODEL" = "Equal Loadings",
                              "SCALAR MODEL" = "Equal Loadings & Thresholds")
all_fits <- all_fits[order(all_fits$gen, all_fits$condition, all_fits$est), ]

write.csv(all_fits, paste0(path, "//05Mplus_output", "//SimulateFits.csv"))
save(all_fits, file = paste0(path, "//05Mplus_output", "//SimulateFits.RData"))